SystemOrganization addCategory: #GODProcessors!

Object subclass: #PCSProcessor
	instanceVariableNames: ''
	classVariableNames: ''
	poolDictionaries: ''
	category: 'GODProcessors'!

!PCSProcessor commentStamp: 'FKP 11/2/2014 19:21' prior: 0!
A PCSProcessor is a class that processes strings, by count of ocurrences.

Instance Variables: none
!

!PCSProcessor methodsFor: 'counting' stamp: 'TD 10/28/2014 09:38'!
countIn: aString
	"return a Dictionary with the number of ocurrences of each word in aString"
	|aCollection  aSet aDictionary|
	aCollection := Array new.
	aSet := Set new.
	aDictionary := Dictionary new. 

	aCollection := aString  subStrings. 
	aSet := aCollection asSet.
	aSet do: [ :aSubString | aDictionary add: (Association key: aSubString value: (aCollection occurrencesOf: aSubString)).].

	 ^aDictionary! !

!PCSProcessor methodsFor: 'counting' stamp: 'TD 10/28/2014 09:39'!
countIn: aString ocurrencesOf: aWord
	"return the number of ocurrences of aWord in aString"
	|aCollection n|
	aCollection := Array new.

	aCollection := aString  subStrings. 
	
	n := aCollection occurrencesOf: aWord.
	^n.! !

PCSProcessor subclass: #PCSStatisticsCalculator
	instanceVariableNames: ''
	classVariableNames: ''
	poolDictionaries: ''
	category: 'GODProcessors'!

!PCSStatisticsCalculator commentStamp: 'FKP 11/2/2014 20:00' prior: 0!
PCSStatisticsCalculator is a class that calculates statistical measures of central tendency and dispersion, for a given generic collection. 

No selector key, the method applies to the absolute values of the collection and its usage is intuitive.

Already with the selector, the measure is calculated on the key value, for each element of the collection.

The key is a code block.

For instance, if you want the average width as an attribute. 
you can use the command:  
  pcs average: aCollection key: [: x | x width].

Now, if you want the average occurrences of a word, you can do:
  pcs average: aCollection key: [: x | x Countin: aword].


Instance Variables: none
!

!PCSStatisticsCalculator methodsFor: 'averages_and_variances' stamp: 'TD 9/27/2014 14:30'!
average: aCollection 
	"compute the average of the collection of real numbers"
	^aCollection average.! !

!PCSStatisticsCalculator methodsFor: 'averages_and_variances' stamp: 'TD 10/7/2014 19:38'!
average: aCollection key: aKey
	"compute the average of key in the collection"
	| aNumberCollection |
	
	aNumberCollection := Bag new: aCollection size.
	
	aCollection do: [ :each | aNumberCollection add: (aKey value: each) ].
	
	^self average: aNumberCollection.! !

!PCSStatisticsCalculator methodsFor: 'tagging' stamp: 'TD 10/28/2014 09:52'!
generateTags: aGODData
    "add tags to a GodData object
	this method has been deprecated
	The feature has been replaced by the class Tagger that uses Information Retrieval technics
	
	To tag all element of a collection, you should do as following
		pcsTagger = PCSTagger new.
		pcsTagger.tagCollection: aGODDataCollection.
	
	To tag an element, you should do as following:
		pcsTagger = PCSTagger new.
		pcsTagger.training: aGODDataCollection.
		pcsTagger.addTagsTo: aGODData.
    "

    |aString aCollection aDictionary|
    
    aDictionary := Dictionary new.
    aCollection := OrderedCollection new.
    aString := String new.
    aString := aGODData content.
    aDictionary := self countIn: aString.
    aCollection := (Dictionary associations asSortedCollection: [:x :y | x value > y value]) 
            collect: [:association | association key].
    
    aGODData tags add: (aCollection at: 1).
    aGODData tags add: (aCollection at: 2).
    aGODData tags add: (aCollection at: 3).! !

!PCSStatisticsCalculator methodsFor: 'initializing' stamp: 'TD 9/27/2014 13:35'!
initialize
	! !

!PCSStatisticsCalculator methodsFor: 'averages_and_variances' stamp: 'TD 10/20/2014 14:15'!
median: aCollection 
	"compute the median of the collection of Number"
	^aCollection median.! !

!PCSStatisticsCalculator methodsFor: 'averages_and_variances' stamp: 'TD 10/20/2014 14:15'!
median: aCollection key: aKey
	"compute the median of this key(k) in the collection(c)"
	|aNumberCollection|
	aNumberCollection := Bag new: aCollection size.
	
	aCollection do: [ :each | aNumberCollection add: (aKey value: each) ].
	
	^self median: aNumberCollection.

	! !

!PCSStatisticsCalculator methodsFor: 'averages_and_variances' stamp: 'TD 10/20/2014 14:31'!
std: aCollection 
	"compute the standard deviation of the collection"
	
	^(self var: aCollection) sqrt.! !

!PCSStatisticsCalculator methodsFor: 'averages_and_variances' stamp: 'TD 10/20/2014 14:39'!
std: aCollection key: aKey
	"compute the standard deviation of key in the collection"
	|aNumberCollection|
	aNumberCollection := Bag new: aCollection size.
	
	aCollection do: [ :each | aNumberCollection add: (aKey value: each) ].
	
	^self std: aNumberCollection.! !

!PCSStatisticsCalculator methodsFor: 'averages_and_variances' stamp: 'TD 10/20/2014 14:42'!
var: aCollection 
	"compute the variance of the collection"
	|average sum|
	
	average := self average: aCollection.
	sum := 0.
	aCollection do: [:aNumber| sum := sum + ((aNumber - average) raisedTo: 2 ) ].
	
	^sum / aCollection size.! !

!PCSStatisticsCalculator methodsFor: 'averages_and_variances' stamp: 'TD 10/20/2014 14:30'!
var: aCollection key: aKey
	"compute the variance of the key in the collection"
	|aNumberCollection|
	aNumberCollection := Bag new: aCollection size.
	
	aCollection do: [ :each | aNumberCollection add: (aKey value: each) ].
	
	^self var: aNumberCollection.! !

PCSProcessor subclass: #PCSTagger
	instanceVariableNames: 'dictIdf minRelevance maxTf minTf numDocuments'
	classVariableNames: ''
	poolDictionaries: ''
	category: 'GODProcessors'!

!PCSTagger commentStamp: 'FKP 11/2/2014 21:31' prior: 0!
A PCSTagger generates tags for GODData using your content. 

To this end, it counts with a variety of methods that enable it: 
do tokenization of strings, calculate the quantities tf-idf, organize the tokens in dictionaries tf-idf, and as the level of significance, generates tags for an element (GODData) or several (several GODData in a collection). 
To generate a tag, tf the respective token must belong to the interval [minTf, maxTf] desired;

The methods training, addTagsTo and tagCollection are specialized to deal with GODData.

How to use

  To tag all element of a collection

      pcsTagger = PCSTagger new.
      pcsTagger tagCollection: aGODDataCollection.

  To tag an element

      pcsTagger = PCSTagger new.
      pcsTagger training: aGODDataCollection.
      pcsTagger addTagsTo: aGODData.

Instance Variables
	
	dictIdf: an dictionary of idf; given a term t it returns the respective idf
	
	maxTf: define the max frequence to be considerated
		   * float in range [0..1]

	minTf: define the min frequence to be considerated
		   * float in range [0..1]
		
	minRelevance: define the min value of relevance that will be accepted as a tag
		   * float in range [0..1]

!

!PCSTagger methodsFor: 'as yet unclassified' stamp: 'TD 11/2/2014 18:35'!
addTagsTo: aGodData
 
	|tags|

	" Get most relevants tokens in aGodData and add it as tags of aGodData "
	
	tags := (self getMoreRelevantsOf: aGodData content) asOrderedCollection.
	aGodData tags: tags.! !

!PCSTagger methodsFor: 'as yet unclassified' stamp: 'TD 10/31/2014 11:29'!
countIn: aBag
	"return a Dictionary with the number of ocurrences of each word in aString"
	|  aSet aDictionary|

	aSet := Set new.
	aDictionary := Dictionary new. 
	aSet := aBag asSet.
	aSet do: [ :aToken | 
		aDictionary add: (Association key: aToken value: (aBag occurrencesOf: aToken))
	].
	 ^aDictionary! !

!PCSTagger methodsFor: 'as yet unclassified' stamp: 'FKP 11/2/2014 20:26'!
createDictIdf: aStringCollection
  	|dictDf aBag|
  	numDocuments := aStringCollection size.
  	dictDf := Dictionary new.
	
  	"generates the dictionary of frequencies of each document"
  	aStringCollection do: [ :aElement |
		aBag := self  preprocess: aElement.
		aBag asSet do: [ :token |
			(dictDf includesKey: token)
				ifTrue:[
					dictDf at: token put: ((dictDf at: token) + 1)
				]
				ifFalse:[
					dictDf at: token put: 1
				].
      		
		]
    	].

  	"converte dict_df into a Inverse Document Frequency dictionary"
  	dictIdf := Dictionary new.
  	dictDf keysAndValuesDo: [ :token :freq |
    		dictIdf at: token put: (( numDocuments / (freq +1)) log)
  	].! !

!PCSTagger methodsFor: 'initialize-release' stamp: 'FCB 11/2/2014 18:20'!
createDictTf: aString
  	|aBag aDictTf|
	
  	" convert aString to an Bag of tokens"
  	aBag := self preprocess: aString.

  	" calculates number of ocourrences of each token"
  	aDictTf := self countIn: aBag.
	
	


   	^aDictTf! !

!PCSTagger methodsFor: 'accessing' stamp: 'TD 10/30/2014 16:23'!
dictIdf
	"get dictIdf"
	^dictIdf.! !

!PCSTagger methodsFor: 'accessing' stamp: 'TD 10/30/2014 16:27'!
dictIdf: value
	"set dictIdf"
	dictIdf := value.! !

!PCSTagger methodsFor: 'as yet unclassified' stamp: 'FCB 11/2/2014 18:26'!
getMoreRelevantsOf: aString
 
	|dictTf tfIdfDict tfIdf minTfIdf maxTfIdf aBag tags|
	
	(self dictIdf isEmpty)
	ifTrue: [
		DictIdfMustNotBeEmpty signal
	].
  	dictTf :=  self createDictTf: aString.
	aBag := self preprocess: aString.
  	"calcula o tfidf de cada termo de aDict"
  	tfIdfDict := Dictionary new.
	
	minTfIdf := 1000.
	maxTfIdf := -1000.
  	dictTf keysAndValuesDo: [ :token :tf |
    		tfIdf := self tfIdf: token withFrequence: tf.
		tfIdfDict at: token put: tfIdf.
		
		"calculates the minTfIdf"
   		 (tfIdf < minTfIdf)
			ifTrue: [
      			minTfIdf := tfIdf.
    			].
		"calculates the maxTfIdf"
   		 (tfIdf > maxTfIdf)
			ifTrue: [
      			maxTfIdf := tfIdf.
    			].  		
		(((tf / aBag size) > self maxTf) | ((tf / aBag size) < self minTf)) 
			ifTrue: [
				dictTf removeKey: token.
			].
			
  	].
	
	"feature scaling, used to bring all values into the range [0,1]"
	 tfIdfDict keysAndValuesDo: [ :token :value |
		tfIdfDict at: token put: ((value - minTfIdf) / (maxTfIdf - minTfIdf)).
  	].
	"calcular minTfIdf e maxTfIdf"
	tags := Bag new.
  	tfIdfDict keysAndValuesDo: [ :token :value |
   		 (value > self minRelevance)
			ifTrue: [
      			tags add: token.
    			]
  	].
  	^tags.! !

!PCSTagger methodsFor: 'initialize-release' stamp: 'TD 11/2/2014 11:01'!
initialize

	"define a standard configuration to the algorithm"
	minRelevance := (0.8).
	minTf := (0.0).
	maxTf := (1.0).
	dictIdf := Dictionary new.! !

!PCSTagger methodsFor: 'initialize-release' stamp: 'TD 10/30/2014 17:13'!
intitialize

	"define a standard configuration to the algorithm"

	minRelevance := (0.8).! !

!PCSTagger methodsFor: 'accessing' stamp: 'TD 10/30/2014 16:24'!
maxTf
	"get maxTf"
	^maxTf. ! !

!PCSTagger methodsFor: 'accessing' stamp: 'TD 10/30/2014 16:27'!
maxTf: value
	"set maxTf"
	maxTf := value. ! !

!PCSTagger methodsFor: 'accessing' stamp: 'TD 10/30/2014 16:28'!
minRelevance
	"get minRelevance"
	^minRelevance. ! !

!PCSTagger methodsFor: 'accessing' stamp: 'TD 10/30/2014 16:28'!
minRelevance: value
	"set minRelevance"
	minRelevance := value. ! !

!PCSTagger methodsFor: 'accessing' stamp: 'TD 10/30/2014 16:27'!
minTf
	"get minTf"
	^minTf. ! !

!PCSTagger methodsFor: 'accessing' stamp: 'TD 10/30/2014 16:28'!
minTf: value
	"set minTf"
	minTf := value. ! !

!PCSTagger methodsFor: 'as yet unclassified' stamp: 'TD 10/28/2014 10:28'!
preprocess: aString
  | aBag |
  "this method realizes comom preprocess methods over the String
	1- tokenizes the String 
    - may remove common words
    - capitalize tokens
    - may do stemming and lemmatization
	anArray := self tokenizer: aS. 
  "
	aBag := self tokenizer: aString. 
	
	
	^aBag! !

!PCSTagger methodsFor: 'as yet unclassified' stamp: 'TD 10/31/2014 09:58'!
tagCollection: aGODDataCollection

	self training: aGODDataCollection.
	aGODDataCollection do: [ :element |
		self addTagsTo: element
  	].! !

!PCSTagger methodsFor: 'as yet unclassified' stamp: 'TD 10/31/2014 12:31'!
tfIdf: aTerm withFrequence: aFrequence
	|idf|
	(dictIdf includesKey: aTerm)
		ifTrue: [	idf:= dictIdf at: aTerm.]
		ifFalse: [idf := ((numDocuments / 1) log). ].
	^(aFrequence * idf).! !

!PCSTagger methodsFor: 'as yet unclassified' stamp: 'TD 10/28/2014 10:28'!
tokenizer: aString
  	|aBag|
	"this method split aString and return a Bag of tokens"
	
	"we decide to use a Bag, once the order of the tokens doesn't matter"
	aBag := (aString subStrings) asBag.
	^aBag.! !

!PCSTagger methodsFor: 'as yet unclassified' stamp: 'TD 10/28/2014 10:51'!
training: aGODDataCollection
	|aStringCollection|
	
  	"it extacts the aGODDataCollection's contents"
	aStringCollection := Bag new.
  	aGODDataCollection do: [ :aElement |
		aStringCollection add: aElement content.
    	].
  	"it trains the algorithm with the given aGODDataCollection contents"
	self createDictIdf: aStringCollection.! !
